@@ -52,7 +52,7 @@ impl Data {
   /// requires info from submessage header. Required iformation is  expect_qos
   /// and expect_payload whish are told on submessage headerflags.
 
-  pub fn deserialize_data(buffer: Bytes, flags: BitFlags<DATA_Flags>) -> io::Result<Data> {
+  pub fn deserialize_data(buffer: &Bytes, flags: BitFlags<DATA_Flags>) -> io::Result<Data> {
     let mut cursor = io::Cursor::new(&buffer);
     let endianness = endianness_flag(flags.bits());
     let map_speedy_err = |p: Error| io::Error::new(io::ErrorKind::Other, p);
@@ -82,7 +82,7 @@ impl Data {
     // always zero.
     let extra_octets = octets_to_inline_qos - rtps_v23_data_header_size;
     // Nevertheless, skip over that extra data, if we are told such exists.
-    cursor.set_position(cursor.position() + extra_octets as u64);
+    cursor.set_position(cursor.position() + u64::from(extra_octets));
 
     let parameter_list = if expect_qos {
       Some(
@@ -94,7 +94,7 @@ impl Data {
     };
 
     let payload = if expect_data {
-      let p = SerializedPayload::from_bytes(buffer.clone().split_off(cursor.position() as usize))?;
+      let p = SerializedPayload::from_bytes(&buffer.clone().split_off(cursor.position() as usize))?;
       Some(p)
     } else {
       None
